# Subset for papers
rm(list = ls())

# Cut-down set of analyses for paper
# Utility functions ####
predict_min_max <- function(x, n=5){
  # Fix tiny rounding errors above and below 0 to 1 on some maps
  for(i in 1:n){
    x[[i]][x[[i]] > 1] <- 1
    x[[i]][x[[i]] < 0] <- 0
  }
  return(x)
}

# Extract maps for Figs 4, 5 and 6 of paper
plot_obs_pred <- function(NVC_pred, NVC_obs, method = c("GLM", "SVM", "RF", "MaxEnt", "MaxLike")){
  # Multiple plots on same scale etc ####
  par(mfrow=c(2,3), oma=c(0,0,0,1))
  for (i in 1:5){
    plot(NVC_pred[[i]], axes = FALSE, box = FALSE,
         main = method[i], zlim=c(0,1))
    plot(NVC_obs, add=TRUE, pch=3, cex = 0.25, lwd = 0.5)
  }  
  par(mfrow=c(1,1), oma=c(0,0,0,0))
}



library(sf)
library(raster)
library(sdm)
library(usdm)
library(dplyr)
library(tidyr)
library(ggplot2)

# Env data ####
DEM_10k <- readRDS("DATA/GIS_data/Rasters10k/DEM_10k.RDS") # used as template for resampling
aspect_10k <- readRDS("DATA/GIS_data/Rasters10k/aspect_10k.RDS")
slope_10k <- readRDS("DATA/GIS_data/Rasters10k/slope_10k.RDS")
EVI_10k <- readRDS("DATA/GIS_data/Rasters10k/EVI_10k.RDS")
rainfall_10k <- readRDS("DATA/GIS_data/Rasters10k/rainfall_10k.RDS")
tempmin_summer_MET <- readRDS("DATA/GIS_data/Rasters10k/tempmin_summer_MET_10k.RDS")
tempmin_winter_MET <- readRDS("DATA/GIS_data/Rasters10k/tempmin_winter_MET_10k.RDS")
tempmax_summer_MET <- readRDS("DATA/GIS_data/Rasters10k/tempmax_summer_MET_10k.RDS")
tempmax_winter_MET <- readRDS("DATA/GIS_data/Rasters10k/tempmax_winter_MET_10k.RDS")


full_stack_10k <- stack(c(aspect_10k,
                          DEM_10k,
                          slope_10k,
                          EVI_10k,
                          rainfall_10k,
                          tempmin_summer_MET,
                          tempmin_winter_MET,
                          tempmax_summer_MET,
                          tempmax_winter_MET))

# Variance inflation factors (VIF)
vifstep(full_stack_10k)
# Drop explan with VIF > 10 which is tempmin_winter_MET
full_stack_10k <- stack(c(aspect_10k,
                          DEM_10k,
                          slope_10k,
                          EVI_10k,
                          rainfall_10k,
                          tempmin_summer_MET,
                          tempmax_summer_MET,
                          tempmax_winter_MET))



# Community data ####
NVC_names <- c("CG9", "CG10", "H8", "H10", "H18", "H21", "M2", "M3",
               "M4", "M16", "M17", "M18", "M20", "M21", "M25",
               "M27", "M29", "M37", "U16", "U17", "U21", "W9")


process_data <- function(nvc_list){
  for(nvc_no in 1:length(nvc_list)){
    print(paste0("Processing NVC ", nvc_list[nvc_no], "..."))
    NVC <- st_read(paste0("DATA/GIS_data/Communities/Shapefiles/cropped/",
                          nvc_list[nvc_no], ".shp" ))
    
    NVC <- st_transform(NVC, 27700)
    NVC_sp <- as(NVC, "Spatial")
    NVC_sp$NVC <- 1
    set.seed(105)
    
    NVC_sdmData <- sdmData(NVC ~  aspect_10k + DEM_10k + slope_10k + EVI_10k + 
                             rainfall_10k + tempmin_summer_MET +
                             tempmax_summer_MET +
                             tempmax_winter_MET,
                           train = NVC_sp, predictors = full_stack_10k,
                           bg = list(n=100, method = 'gRandom', remove = TRUE))
    set.seed(006)
    NVC_sdm <- sdm(NVC ~ aspect_10k + DEM_10k + slope_10k + EVI_10k + 
                     rainfall_10k + tempmin_summer_MET + 
                     tempmax_summer_MET + tempmax_winter_MET,
                   data = NVC_sdmData,
                   methods = c('glm', 'svm', 'rf', 'entropy', 'maxlike'),
                   replication = 'boot', n = 100)

    predict_NVC_mean <- predict(NVC_sdm, full_stack_10k,
                                filename = "sdm_prediction.img",
                                overwrite = TRUE,
                                mean = TRUE)
    
    predict_NVC_mean <- predict_min_max(predict_NVC_mean)
    saveRDS(NVC, paste0("DATA/RDS/", nvc_list[nvc_no], ".RDS"))
    saveRDS(NVC_sp, paste0("DATA/RDS/", nvc_list[nvc_no], "_sp.RDS"))
    saveRDS(NVC_sdmData, paste0("DATA/RDS/", nvc_list[nvc_no], "_sdmData.RDS"))
    saveRDS(NVC_sdm, paste0("DATA/RDS/", nvc_list[nvc_no], "_sdm.RDS"))
    saveRDS(predict_NVC_mean, paste0("DATA/RDS/", nvc_list[nvc_no], "_predict.RDS"))
    }
}



process_data(NVC_names)

# Compare model performance with number of occurrence records ####
# Occurrences. Note, this is lower than the number of records as duplicate 10kms
nvc_perf <- data.frame(matrix(nrow=length(NVC_names), ncol=16))
colnames(nvc_perf) <- c("count","AUC_GLM","AUC_MaxEnt","AUC_MaxLike","AUC_RF",
                        "AUC_SVM","TSS_GLM","TSS_MaxEnt","TSS_MaxLike","TSS_RF",
                        "TSS_SVM","KPP_GLM","KPP_MaxEnt","KPP_MaxLike","KPP_RF",
                        "KPP_SVM")
for(nvc_no in 1:length(NVC_names)){
  NVC <- st_read(paste0("DATA/GIS_data/Communities/Shapefiles/cropped/",
                        NVC_names[nvc_no], ".shp" ))
  NVC <- st_transform(NVC, 27700)
  NVC_cnt <- nrow(distinct(NVC))
  nvc_perf[nvc_no, 1] <- NVC_cnt
  
  # Model performance data AUC and TSS
  NVC_sdm <- readRDS(paste0("DATA/RDS/", NVC_names[nvc_no], "_sdm.RDS"))
  NVC_eval <- getEvaluation(NVC_sdm, stat = c('TSS', 'Kappa', 'AUC'), opt = 1)
  methods <- rep(c('GLM', 'SVM', 'RF', 'MaxEnt', 'MaxLike'), each = 10)
  NVC_eval <- cbind(NVC_eval, methods)
  
  NVC_eval_tbl <- NVC_eval %>%
    rename(`Model` = methods) %>% 
    group_by(`Model`) %>% 
    summarise(AUC = mean(AUC), TSS = mean(TSS), Kappa = mean(Kappa)) %>% 
    pivot_wider(names_from=1, values_from=2:4)
  
  this_nvc_perf <- c(NVC_cnt, NVC_eval_tbl)
  nvc_perf[nvc_no,] <- this_nvc_perf
}

nvc_perf_lng <- nvc_perf %>% 
  pivot_longer(cols=2:16) %>% 
  mutate(metric = substr(name, 1, 3), model = substr(name, 5, nchar(name)))

nvc_perf_tss <- filter(nvc_perf_lng, metric == "TSS") %>% 
  group_by(count) %>% 
  summarise(mean_metric = mean(value))
ggplot(nvc_perf_tss, aes(x = count, y = mean_metric)) +
  geom_point() +
  # geom_smooth(method = lm) +
  xlab("Number of 10 km occurrence records") +
  ylab("TSS") +
  theme_classic()
summary(lm(mean_metric ~ count, data = nvc_perf_tss))

nvc_perf_auc <- filter(nvc_perf_lng, metric == "AUC") %>% 
  group_by(count) %>% 
  summarise(mean_metric = mean(value))
ggplot(nvc_perf_auc, aes(x = count, y = mean_metric)) +
  geom_point() +
  # geom_smooth(method = lm) +
  xlab("Number of 10 km occurrence records") +
  ylab("AUC") +
  theme_classic()
summary(lm(mean_metric ~ count, data = nvc_perf_auc))

nvc_perf_kappa <- filter(nvc_perf_lng, metric == "KPP") %>% 
  group_by(count) %>% 
  summarise(mean_metric = mean(value))
ggplot(nvc_perf_kappa, aes(x = count, y = mean_metric)) +
  geom_point() +
  # geom_smooth(method = lm) +
  xlab("Number of 10 km occurrence records") +
  ylab("Kappa") +
  theme_classic()
summary(lm(mean_metric ~ count, data = nvc_perf_kappa))

# Mean model performance
library(dplyr)
nvc_list <- c("CG9", "CG10", "H8", "H10", "H18", "H21", "M2", "M3",
              "M4", "M16", "M17", "M18", "M20", "M21", "M25",
              "M27", "M29", "M37", "U16", "U17", "U21", "W9")
NVC_var_df <- NULL #data.frame(matrix(nrow=10*length(nvc_list), ncol=5))
for(nvc_no in 1:length(nvc_list)){
  NVC_sdm <- readRDS(paste0("DATA/RDS/", nvc_list[nvc_no], "_sdm.RDS"))
  NVC_var <- getVarImp(NVC_sdm, wtest = 'training', id=1:50)
  NVC_var_plt <- plot(NVC_var, 'cor')
  NVC_var_dta <- NVC_var_plt$data
  NVC_var_dta <- mutate(NVC_var_dta, nvc = nvc_list[nvc_no])
  NVC_var_df <- rbind(NVC_var_df, NVC_var_dta)
}

# Fig. 3
NVC_var_df %>% 
  group_by(variables) %>% 
  summarise(mean = mean(corTest),
            sd = sd(corTest),
            n = n(),
            se = sd / sqrt(n),
            ci95 = se * qt(.975, n - 1)) %>% 
  ggplot(aes(x = variables, y = mean)) +
    geom_bar(stat = "identity", fill = "grey60") +
  geom_errorbar(aes(ymin = mean - ci95, ymax = mean + ci95),
                size = 0.3,
                width = 0.2) +
  scale_x_discrete(labels = c("aspect","elevation","EVI",
                              "annual rainfall",
                              "slope","summer max temp",
                              "winter max temp","summer min temp")) +
  ylab("Variable importance (1 - cor)") +
  theme_classic() +
  theme(axis.title.x = element_blank(),
        axis.text.x  = element_text(angle=60, vjust=0.5, size=10)) 

# Add info on performance statistics per NVC
nvc_perf_lng <- nvc_perf_lng %>% 
  bind_cols(rep(nvc_list, each=15)) %>% 
  rename(nvc = ...6)

# Performance boxplots overall
ggplot(nvc_perf_lng, aes(x=model, y=value)) +
  geom_boxplot() +
  facet_wrap( ~ metric, labeller=labeller(metric = c(AUC = "a) AUC",
                                                     KPP = "b) Kappa",
                                                     TSS = "c) TSS"))) +
  ylab("Model performance") +
  theme_classic() +
  theme(axis.title.x = element_blank())

# Fig. 2
library(cowplot)
p1 <- nvc_perf_lng %>% 
  filter(metric == "AUC") %>% 
  ggplot(aes(x = model, y = value)) +
    geom_boxplot() +
    ylim(c(0, 1)) +
    ylab("AUC") +
    theme_classic() +
    theme(axis.title.x = element_blank())
p2 <- nvc_perf_lng %>% 
  filter(metric == "TSS") %>% 
  ggplot(aes(x = model, y = value)) +
  geom_boxplot() +
  ylim(c(0, 1)) +
  ylab("TSS") +
  theme_classic() +
  theme(axis.title.x = element_blank())
p3 <- nvc_perf_lng %>% 
  filter(metric == "KPP") %>% 
  ggplot(aes(x = model, y = value)) +
  geom_boxplot() +
  ylim(c(0, 1)) +
  ylab("Kappa") +
  theme_classic() +
  theme(axis.title.x = element_blank())
plot_grid(p1, p2, p3, labels = c("a", "b", "c"))


# Check on overall performance summaries for communities
# High scores (0.966): U10, U17, W19, H19, M11
# Median scores (0.721; 0.818 for RF): CG10, M1, M3, M4, H18
# Low scores (0.0857): H8, M16, M21, M27,
nvc_perf_lng %>% 
  filter(metric == "TSS") %>% 
  arrange(desc(value)) %>% 
  View()

# Summary info
nvc_perf_lng %>% 
  group_by(metric) %>% 
  summarise(median_score = median(value), min_score = min(value), max_score = max(value))

nvc_perf_lng %>%
  filter(metric == "TSS") %>% 
  group_by(model) %>% 
  summarise(mean_score = mean(value), median_score = median(value), min_score = min(value), max_score = max(value))

nvc_perf_lng %>%
  filter(metric == "TSS" & model == "RF") %>% 
  View()

nvc_perf_lng %>%
  filter(metric == "TSS" & model == "RF") %>% 
  summarise(median_score = median(value), min_score = min(value), max_score = max(value))

# Individual community info
nvc_perf_lng %>%
  filter(nvc == "U17" & metric == "TSS") %>% 
  arrange(value)
nvc_perf_lng %>%
  filter(nvc == "H18" & metric == "TSS") %>% 
  arrange(value)
nvc_perf_lng %>%
  filter(nvc == "M27" & metric == "TSS") %>% 
  arrange(value)


# Fig. 4 U17
NVC_sp <- readRDS(paste0("DATA/RDS/U17_sp.RDS"))
predict_NVC_mean <- readRDS("DATA/RDS/U17_predict.RDS")
png(file = "Figures/U17_map.png", width = 8, height = 5, res = 600, units = "in")
plot_obs_pred(predict_NVC_mean, NVC_sp)
dev.off()
# Fig. 5 H18
NVC_sp <- readRDS(paste0("DATA/RDS/H18_sp.RDS"))
predict_NVC_mean <- readRDS("DATA/RDS/H18_predict.RDS")
png(file = "Figures/H18_map.png", width = 8, height = 5, res = 600, units = "in")
plot_obs_pred(predict_NVC_mean, NVC_sp)
dev.off()
# Fig. 6 M27
NVC_sp <- readRDS(paste0("DATA/RDS/M27_sp.RDS"))
predict_NVC_mean <- readRDS("DATA/RDS/M27_predict.RDS")
png(file = "Figures/M27_map.png", width = 8, height = 5, res = 600, units = "in")
plot_obs_pred(predict_NVC_mean, NVC_sp)
dev.off()
